@@ -2,6 +2,7 @@ import datetime
 import sys
 from copy import copy
 import sqlite3
+from io import BytesIO
 import xlrd
 from typing import Callable, Union, List, Any
 import os
@@ -9,8 +10,7 @@ import shutil
 import xlsxwriter
 import csv
 import json
-from excel_magic.chromium_downloader import check_chromium, download_chromium
-import requests
+from PIL import Image
 
 __all__ = ['Sheet', 'Dataset', 'open_file']
 
@@ -106,6 +106,12 @@ class Cell:
                 return self.value is other
 
 
+class ImageCell(Cell):
+    def __init__(self, data: BytesIO, value=''):
+        super().__init__()
+        self.data = data
+        self.value = value
+
 class Sheet:
     def __init__(self, path: str, sheet: Union[xlrd.sheet.Sheet, str] = ''):
         self.fields = []
@@ -186,7 +192,7 @@ class Sheet:
 
         return data_list
 
-    def append_row(self, content: Union[dict, List[str]]) -> None:
+    def append_row(self, content: Union[dict, List[Union[str, Cell]]]) -> None:
         new_row = {}
         if isinstance(content, dict):
             for field in self.fields:
@@ -199,7 +205,11 @@ class Sheet:
                     new_row[field] = Cell('')
         elif isinstance(content, list):
             for i in range(len(self.fields)):
-                new_row[self.fields[i]] = Cell(content[i])
+                if isinstance(content[i], Cell):
+                    new_row[self.fields[i]] = content[i]
+                else:
+                    new_row[self.fields[i]] = Cell(content[i])
+
             if len(content) < len(self.fields):
                 for i in range(len(content) - 1, len(self.fields)):
                     new_row[self.fields[i]] = Cell('')
@@ -269,30 +279,6 @@ class Sheet:
         with open(out, 'w') as f:
             json.dump(data, f)
 
-    def to_pdf(self, out='out.pdf', config=''):
-        if not (os.path.isfile('render-pea') or os.path.isfile('render-pea.exe')):
-            # raise FileNotFoundError('render-pea plugin is not present')
-            if sys.platform == 'darwin':
-                filename = 'render-pea'
-            else:
-                filename = 'render-pea.exe'
-            url = '' + sys.platform + filename
-            file = requests.get(url)
-            with open(filename, 'wb') as f:
-                f.write(file.content)
-        if not check_chromium():
-            download_chromium()
-
-        if sys.platform.startswith('win'):
-            cmd = f'render-pea.exe "{self.filename}" --name "{self.name}" --out "{out}"'
-        else:
-            cmd = f'./render-pea "{self.filename}" --name "{self.name}" --out "{out}"'
-        if config != '':
-            cmd += f' --config "{config}"'
-        print(cmd)
-        a = os.popen(cmd)
-        print(a)
-
     def split_rows(path: str, row_count: int, name_by: str):
         filenames = {}
 
@@ -494,7 +480,7 @@ class Dataset:
     def remove_sheet_by_index(self, index: int):
         pass
 
-    def save(self, backup = True):
+    def save(self, backup = True, row_height = 0, col_width = 0):
         # make backup & delete
         if os.path.isfile(os.path.join(self.path, self.filename)) and backup:
             shutil.copy(os.path.join(self.path, self.filename), os.path.join(self.path, self.backup_name))
@@ -516,7 +502,22 @@ class Dataset:
                     if isinstance(data.value, datetime.datetime):
                         sheet.write(pointer.row, pointer.col, str(data.value), workbook.add_format(data.attr()))
                     else:
-                        sheet.write(pointer.row, pointer.col, data.value, workbook.add_format(data.attr()))
+                        if isinstance(data, ImageCell):
+                            sheet.insert_image(pointer.row, pointer.col, data.value, {'image_data': data.data, 'y_offset': 10, 'x_offset': 10})
+                            data.data.seek(0)
+                            img: Image.Image = Image.open(data.data)
+                            width, height = img.size
+                            if row_height == 0:
+                                sheet.set_row(pointer.row, height)
+                            else:
+                                sheet.set_row(pointer.row, row_height)
+                            if col_width != 0:
+                                sheet.set_column(pointer.col, pointer.col, (col_width / 8))
+                            else:
+                                sheet.set_column(pointer.col, pointer.col, (width / 8))
+
+                        else:
+                            sheet.write(pointer.row, pointer.col, data.value, workbook.add_format(data.attr()))
                     pointer.next_col()
                 pointer.next_row()
         workbook.close()
